@@ -35,27 +35,13 @@ WARNING = 'warning'
 ERROR = 'error'
 CRITICAL = 'critical'
 
-
-class FormatError(Exception):
-    pass
-
-
 class Logger(object):
     """
     Logger([stream[, threshold[, format[]]]) -> Logger object
 
     Provides a basic logging object.
-
-    Args:
-        stream: string - full file path OR object - object implementing
-            write and flush methods
-
-        threshold: simplelogger constant - minimum level to log
-            (eg simplelogger.INFO)
-        format: string - custom logging format. See set_format for more
-            details
     """
-    def __init__(self, stream=None, threshold=None, format=None):
+    def __init__(self, stream=None, threshold=None, format=None, log_errors=True):
         """
         Args:
             stream: string - full file path OR object - object implementing
@@ -66,14 +52,17 @@ class Logger(object):
 
             format: string - custom logging format. See set_format for
                 more details
+
+            log_errors: boolean - set whether or not errors are logged
+                to the stream object
         """
 
-        self._levels = ['debug', 'info', 'warning', 'error', 'critical']
+        self._levels = [DEBUG, INFO, WARNING, ERROR, CRITICAL]
 
         if threshold == None:
-            self._threshold = 'info'
+            self.set_threshold(INFO)
         else:
-            self._threshold = threshold
+            self.set_threshold(threshold)
 
         if format is None:
             self._format = '{datetime}: {level}: {text}'
@@ -82,12 +71,15 @@ class Logger(object):
 
         if stream is not None:
             if isinstance(stream, basestring):
-                if not os.path.exists(os.path.dirname(stream)):
+                if not os.path.isdir(os.path.dirname(stream)):
                     os.makedirs(os.path.dirname(stream))
 
                 try:
                     self._stream = open(stream, 'a+')
-                    sys.stderr = open(stream, 'a+')
+
+                    if log_errors:
+                        sys.stderr = self._stream
+
                 except IOError as e:
                     raise IOError("Could not open {stream}".format(stream=e.filename))
             else:
@@ -95,6 +87,10 @@ class Logger(object):
                     stream.write('')
                     stream.flush()
                     self._stream = stream
+
+                    if log_errors:
+                        sys.stderr = self._stream
+
                 except AttributeError:
                     raise AttributeError("Provided stream object is invalid, must implement write and flush methods")
                 except IOError:
@@ -110,14 +106,7 @@ class Logger(object):
         self._stream.flush()
 
     def _above_threshold(self, level):
-        for j in range(0,len(self._levels)-1):
-            if self._levels[j] == self._threshold:
-                break;
-        for i in range(0,len(self._levels)-1):
-            if self._levels[i] == level:
-                break
-
-        if i >= j:
+        if level in self._loggable_levels or level not in self._levels:
             return True
         else:
             return False
@@ -130,6 +119,12 @@ class Logger(object):
         """
         self._threshold = threshold
 
+        for i in range(len(self._levels)):
+            if self._levels[i] == self._threshold:
+                break;
+
+        self._loggable_levels = self._levels[i:]
+
     def set_format(self, format):
         """
         Set custom log format
@@ -147,9 +142,12 @@ class Logger(object):
         """
         Logs text to file or standard _output
 
-        For custom logging levels. It is recommended to use provided the provided
-        functions for logging various levels:
+        For custom logging levels. It is recommended to use provided
+        the provided functions for logging various levels:
         debug(), info(), warning(), error(), and critical().
+        
+        Custom logging levels are always over the threshold, and therefore
+        always logged.
 
         Args:
             text: string - message to be displayed
@@ -180,7 +178,7 @@ class Logger(object):
         Args:
             text: string - text to log
         """
-        self.log("debug",text)
+        self.log(DEBUG,text)
 
     def info(self, text):
         """
@@ -191,7 +189,7 @@ class Logger(object):
         Args:
             text: string - text to log
         """
-        self.log("info",text)
+        self.log(INFO,text)
 
     def warning(self, text):
         """
@@ -202,7 +200,7 @@ class Logger(object):
         Args:
             text: string - text to log
         """
-        self.log("warning",text)
+        self.log(WARNING,text)
 
     def error(self, text):
         """
@@ -213,7 +211,7 @@ class Logger(object):
         Args:
             text: string - text to log
         """
-        self.log("error",text)
+        self.log(ERROR,text)
 
     def critical(self, text):
         """
@@ -224,4 +222,4 @@ class Logger(object):
         Args:
             text: string - text to log
         """
-        self.log("critical",text)
+        self.log(CRITICAL,text)
